<?php

require_once SERVICETRADE_PORTAL_PLUGIN_ROOT_DIR . 'includes/models/class-api-credentials.php';
require_once SERVICETRADE_PORTAL_PLUGIN_ROOT_DIR . 'includes/models/class-servicetrade-options.php';
require_once SERVICETRADE_PORTAL_PLUGIN_ROOT_DIR . 'admin/controllers/class-page-users-controller.php';
require_once SERVICETRADE_PORTAL_PLUGIN_ROOT_DIR . 'admin/controllers/class-page-add-user-controller.php';
require_once SERVICETRADE_PORTAL_PLUGIN_ROOT_DIR . 'includes/class-servicetrade-portal-servicetrade-wrapper.php';

/**
 * The admin-specific functionality of the plugin.
 *
 * Defines the plugin name, version, and two examples hooks for how to
 * enqueue the admin-specific stylesheet and JavaScript.
 *
 * @link       http://www.servicetrade.com
 * @since      2.0.0
 * @package    Servicetrade_Service_Portal
 * @subpackage Servicetrade_Service_Portal/admin
 * @author     ServiceTrade <support@servicetrade.com>
 */
class Servicetrade_Service_Portal_Admin {
	private $plugin_name = 'servicetrade-service-portal';
	/**
	 * The section id to be used for configuration section.
	 *
	 * @since  2.0.0
	 * @access 	private
	 * @var  	string 		$section_configuration_id Immutable id for the api section
	 */
	private static $section_configuration_id = 'servicetrade_portal_configuration';

	/**
	 * The section id to be used for the features section.
	 *
	 * @since  2.0.0
	 * @access 	private
	 * @var  	string 		$section_features_id Immutable id for the general section
	 */
	private static $section_features_id = 'servicetrade_portal_features';


	/**
	 * The ID of this plugin.
	 *
	 * @since    1.0.0
	 * @access   private
	 * @var      string    $Servicetrade_Service_Portal    The ID of this plugin.
	 */
	private $Servicetrade_Service_Portal;

	/**
	 * The version of this plugin.
	 *
	 * @since    1.0.0
	 * @access   private
	 * @var      string    $version    The current version of this plugin.
	 */
	private $version;

	private $store;

	/**
	 * Initialize the class and set its properties.
	 *
	 * @since    1.0.0
	 * @param      string    $Servicetrade_Service_Portal       The name of this plugin.
	 * @param      string    $version    The version of this plugin.
	 */
	public function __construct( $Servicetrade_Service_Portal, $version ) {

		$this->Servicetrade_Service_Portal = $Servicetrade_Service_Portal;
		$this->version = $version;

	}

	/**
	 * Register the stylesheets for the admin area.
	 *
	 * @since    1.0.0
	 */
	public function enqueue_styles() {

		/**
		 * This function is provided for demonstration purposes only.
		 *
		 * An instance of this class should be passed to the run() function
		 * defined in Servicetrade_Service_Portal_Loader as all of the hooks are defined
		 * in that particular class.
		 *
		 * The Servicetrade_Service_Portal_Loader will then create the relationship
		 * between the defined hooks and the functions defined in this
		 * class.
		 */

		wp_enqueue_style(  'servicetrade-service-portal-jquery-datatables-css', plugin_dir_url( __FILE__ ) . 'css/jquery.dataTables.min.css', array(), $this->version, 'all' );
		wp_enqueue_style( 'servicetrade-service-portal-admin-css', plugin_dir_url( __FILE__ ) . 'css/servicetrade-portal-admin.css', array(), $this->version, 'all' );

	}

	/**
	 * Register the JavaScript for the admin area.
	 *
	 * @since    1.0.0
	 */
	public function enqueue_scripts() {
		/**
		 * This function is provided for demonstration purposes only.
		 *
		 * An instance of this class should be passed to the run() function
		 * defined in Servicetrade_Service_Portal_Loader as all of the hooks are defined
		 * in that particular class.
		 *
		 * The Servicetrade_Service_Portal_Loader will then create the relationship
		 * between the defined hooks and the functions defined in this
		 * class.
		 */

		wp_enqueue_script('jquery'); // Probably not needed at all

		wp_enqueue_script( 'servicetrade-service-portal-jquery-datatables', plugin_dir_url( __FILE__ ) . 'js/jquery.dataTables.min.js', array( 'jquery' ), $this->version, false );
		// no longer needs to be included; WP includes jquery-ui automatically
		//wp_enqueue_script( 'servicetrade-service-portal-jquery-ui', plugin_dir_url( __FILE__ ) . 'js/jquery-ui.min.js', array( 'jquery' ), $this->version, false );
		wp_enqueue_script( 'servicetrade-service-portal-jquery-succinct', plugin_dir_url( __FILE__ ) . 'js/jquery.succinct.min.js', array( 'jquery' ), $this->version, false );
		wp_enqueue_script( 'servicetrade-service-portal-jquery-popupoverlay', plugin_dir_url( __FILE__ ) . 'js/jquery.popupoverlay.js', array( 'jquery' ), $this->version, false );

		// TODO - Potentially not used
		wp_enqueue_script( 'servicetrade-service-portal-admin', plugin_dir_url( __FILE__ ) . 'js/servicetrade-portal-admin.js', array( 'jquery' ), NULL, false);

		wp_enqueue_script( 'servicetrade-service-portal-admin-add-user', plugin_dir_url( __FILE__ ) . 'js/servicetrade-portal-admin-add-user.js', array( 'jquery' ), NULL, false);
		wp_enqueue_script( 'servicetrade-service-portal-admin-users', plugin_dir_url( __FILE__ ) . 'js/servicetrade-portal-admin-users.js', array( 'jquery' ), NULL, false);
		// TODO - Delete file, then remove reference
		// wp_enqueue_script( 'servicetrade-service-portal-admin-data-table', plugin_dir_url( __FILE__ ) . '../shared/js/servicetrade-portal-admin-data-table.js', array( 'jquery','servicetrade-portal-jquery-datatables' ), NULL, false);
	}


	/**
	 * Register the settings used by the admin.
	 *
	 * @since    2.0.0
	 */
	public function register_settings() {
		$this->register_settings_configuration();
		$this->register_settings_features();
	}

	/**
	 * Register the general settings used by the admin.
	 *
	 * @since    2.0.0
	 */
	public function register_settings_features() {

		add_settings_section(
				Servicetrade_Service_Portal_Admin::$section_features_id,
				__( 'Features', 'servicetrade' ),
				array( $this, 'servicetrade_section_general_cb' ),
				$this->plugin_name
			);

		add_settings_field(
			Servicetrade_Models_ServiceTrade_Options::$request_service_key,
			__( 'Request Service', 'servicetrade' ),
			array( $this, 'servicetrade_request_service_cb' ),
			$this->plugin_name,
			Servicetrade_Service_Portal_Admin::$section_features_id,
			array( 'label_for' => Servicetrade_Models_ServiceTrade_Options::$request_service_key ));
		register_setting( $this->plugin_name, Servicetrade_Models_ServiceTrade_Options::$request_service_key, array( $this,'servicetrade_sanitize_request_service' ) );

		add_settings_field(
			Servicetrade_Models_ServiceTrade_Options::$show_canceled_appointments_key,
			__( 'Show Canceled Appointments', 'servicetrade' ),
			array( $this, 'servicetrade_show_canceled_appointments_cb' ),
			$this->plugin_name,
			Servicetrade_Service_Portal_Admin::$section_features_id,
			array( 'label_for' => Servicetrade_Models_ServiceTrade_Options::$show_canceled_appointments_key ));
		register_setting( $this->plugin_name, Servicetrade_Models_ServiceTrade_Options::$show_canceled_appointments_key, array( $this,'servicetrade_sanitize_show_canceled_appointments' ) );


	}


	/**
	 * We are setting the access token to null to force a reauthentication.
	 * And because it's so slow, we attempt to do that here.
	 */
	public function on_api_username_password_changed() {
		// error_log('on_api_username_password_changed');
		Servicetrade_Models_API_Credentials::set_access_token(null);

		if(Servicetrade_Wrapper::hasCredentials()) {
			// error_log('on_api_username_password_changed - has credentials');
			$baseUrl = SERVICETRADE_API_URL;
			// error_log('on_api_username_password_changed - base url ' . $baseUrl);

			$api = new Servicetrade_Wrapper($baseUrl);
			// error_log('on_api_username_password_changed - connecting ');
			$api->connect();
			// error_log('on_api_username_password_changed - done');
		}
	}
	/**
	 * Register the settings used by the api.
	 *
	 * @since    2.0.0
	 */
	private function register_settings_configuration() {

		add_settings_section(
				Servicetrade_Service_Portal_Admin::$section_configuration_id,
				__( 'Configuration', 'servicetrade' ),
				array( $this, 'servicetrade_section_api_cb' ),
				$this->plugin_name
			);



		// Registering username
		add_settings_field(
			Servicetrade_Models_API_Credentials::$username_key,
			__( 'Name', 'servicetrade' ),
			array( $this, 'servicetrade_username_cb' ),
			$this->plugin_name,
			Servicetrade_Service_Portal_Admin::$section_configuration_id,
			array( 'label_for' => Servicetrade_Models_API_Credentials::$username_key ));

		register_setting( $this->plugin_name,
											Servicetrade_Models_API_Credentials::$username_key,
											array( $this,'servicetrade_sanitize_username' ) );

		// Registering Password
		add_settings_field(
			Servicetrade_Models_API_Credentials::$password_key,
			__( 'Password', 'servicetrade' ),
			array( $this, 'servicetrade_password_cb' ),
			$this->plugin_name,
			Servicetrade_Service_Portal_Admin::$section_configuration_id,
			array( 'label_for' => Servicetrade_Models_API_Credentials::$password_key ));

		register_setting( $this->plugin_name,
											Servicetrade_Models_API_Credentials::$password_key,
											array( $this,'servicetrade_sanitize_password' ) );

		add_settings_field(
			Servicetrade_Models_ServiceTrade_Options::$admin_email_key,
			__( 'Admin Email', 'servicetrade' ),
			array( $this, 'servicetrade_admin_email_cb' ),
			$this->plugin_name,
			Servicetrade_Service_Portal_Admin::$section_configuration_id,
			array( 'label_for' => Servicetrade_Models_ServiceTrade_Options::$admin_email_key ));
		register_setting( $this->plugin_name,
											Servicetrade_Models_ServiceTrade_Options::$admin_email_key,
											array( $this,'servicetrade_sanitize_admin_email' ) );

		add_settings_field(
			Servicetrade_Models_ServiceTrade_Options::$portal_page_url_key,
			__( 'Portal Page Name', 'servicetrade' ),
			array( $this, 'servicetrade_portal_page_url_cb' ),
			$this->plugin_name,
			Servicetrade_Service_Portal_Admin::$section_configuration_id,
			array( 'label_for' => Servicetrade_Models_ServiceTrade_Options::$portal_page_url_key ));
		register_setting( $this->plugin_name,
											Servicetrade_Models_ServiceTrade_Options::$portal_page_url_key,
											array( $this,'servicetrade_sanitize_portal_page_url' ) );

		add_settings_field(
			Servicetrade_Models_ServiceTrade_Options::$timezone_key,
			__( 'Timezone', 'servicetrade' ),
			array( $this, 'servicetrade_timezone_cb' ),
			$this->plugin_name,
			Servicetrade_Service_Portal_Admin::$section_configuration_id,
			array( 'label_for' => Servicetrade_Models_ServiceTrade_Options::$timezone_key ));
		register_setting( $this->plugin_name,
											Servicetrade_Models_ServiceTrade_Options::$timezone_key,
											array( $this,'servicetrade_sanitize_timezone' ) );

	}

/**
	 * Sanitize the username value before being saved to database
	 *
	 * @param  string $timezone $_POST value
	 * @since  2.0.0
	 * @return string           Sanitized value
	 */
	public function servicetrade_sanitize_username( $username ) {
		return $username;
	}

	public function servicetrade_sanitize_request_service( $request_service ) {
		return $request_service;
	}

	/**
	 * Sanitize the password value before being saved to database
	 *
	 * @param  string $timezone $_POST value
	 * @since  2.0.0
	 * @return string           Sanitized value
	 */
	public function servicetrade_sanitize_password( $password ) {
		return Servicetrade_Models_API_Credentials::encrypt_password($password);
	}


	public function servicetrade_sanitize_portal_page_url( $url ) {
		return $url;
	}

/**
	 * Sanitize the admin email value before being saved to database
	 *
	 * @param  string $admin_email $_POST value
	 * @since  1.0.0
	 * @return string           Sanitized value
	 */
	public function servicetrade_sanitize_admin_email( $admin_email ) {
		return $admin_email;
	}

/**
	 * Sanitize the text timezone value before being saved to database
	 *
	 * @param  string $timezone $_POST value
	 * @since  1.0.0
	 * @return string           Sanitized value
	 */
	public function servicetrade_sanitize_timezone( $timezone ) {
		return $timezone;
	}

	/**
	 * Render the text for the api section
	 *
	 * @since  2.0.0
	 */
	public function servicetrade_section_api_cb() {
		echo '<p>' . __( 'Please enter your API Credentials.', 'servicetrade' ) . '</p>';
	}




	/**
	 * Render the text for the general section
	 *
	 * @since  2.0.0
	 */
	public function servicetrade_section_general_cb() {
		echo '<p>' . __( 'Please choose the features you wish to enable.', 'servicetrade' ) . '</p>';
	}

	/**
	 * Render the text input field for username option
	 *
	 * @since  2.0.0
	 */
	public function servicetrade_username_cb() {
		$username = Servicetrade_Models_API_Credentials::get_username();
		?>
			<fieldset>
				<label>
					<input type="text" name="<?php echo 'servicetrade_portal_username' ?>" id="<?php echo 'servicetrade_portal_username' ?>" value="<?php echo $username ?>">
					<?php _e( 'The username you received from ServiceTrade.', 'servicetrade' ); ?>
				</label>
			</fieldset>
		<?php
	}

	/**
	 * Render the text input field for password option
	 *
	 * @since  2.0.0
	 */
	public function servicetrade_password_cb() {
		$decryptedPassword = Servicetrade_Models_API_Credentials::get_password();

		?>
			<fieldset>
				<label>
					<input type="password" name="<?php echo 'servicetrade_portal_password' ?>" id="<?php echo 'servicetrade_portal_password' ?>" value="<?php echo $decryptedPassword ?>">
					<?php _e( 'The Password you received from ServiceTrade.', 'servicetrade' ); ?>
				</label>
			</fieldset>
		<?php
	}

	/**
	 * Render the text input field for admin email option
	 *
	 * @since  2.0.0
	 */
	public function servicetrade_admin_email_cb() {
		$admin_email = Servicetrade_Models_ServiceTrade_Options::get_admin_email();
		?>
			<fieldset>
				<label>
					<input type="email" name="<?php echo 'servicetrade_portal_admin_email' ?>" id="<?php echo 'servicetrade_portal_admin_email' ?>" value="<?php echo $admin_email ?>">
					<?php _e( 'Your admin email.', 'servicetrade' ); ?>
				</label>
			</fieldset>
		<?php
	}

	public function servicetrade_portal_page_url_cb() {
		$url = Servicetrade_Models_ServiceTrade_Options::get_portal_page_url();
		?>
			<fieldset>
				<label>
					<input type="text" name="<?php echo 'servicetrade_portal_page_url' ?>" id="<?php echo 'servicetrade_portal_page_url' ?>" value="<?php echo $url ?>">
					<?php _e( 'The name of the page where you embedded the ServiceTrade Service Portal shortcode.', 'servicetrade-service-portal' ); ?>
				</label>
			</fieldset>
		<?php
	}

	public function servicetrade_request_service_cb() {
		$request_service = Servicetrade_Models_ServiceTrade_Options::get_request_service();
		?>
			<fieldset>
				<label>

				<input name="<?php echo 'servicetrade_portal_request_service' ?>"
									 type="checkbox"
									 id="<?php echo 'servicetrade_portal_request_service' ?>"
									 value="1" <?php checked( '1', $request_service ); ?> />


					<?php _e( 'Customers can request a service from asset and location pages.', 'servicetrade' ); ?>
				</label>
			</fieldset>
		<?php
	}

	public function servicetrade_show_canceled_appointments_cb() {
		$show_canceled_appointments = Servicetrade_Models_ServiceTrade_Options::get_show_canceled_appointments();
		?>
			<fieldset>
				<label>

				<input name="<?php echo 'servicetrade_portal_show_canceled_appointments' ?>"
									 type="checkbox"
									 id="<?php echo 'servicetrade_portal_show_canceled_appointments' ?>"
									 value="1" <?php checked( '1', $show_canceled_appointments ); ?> />


					<?php _e( 'Customers can see canceled appointments in job lists.', 'servicetrade' ); ?>
				</label>
			</fieldset>
		<?php
	}

	/**
	 * Render the combobox field for timezone option
	 *
	 * @since  1.0.0
	 */
	public function servicetrade_timezone_cb() {
		$timezone = Servicetrade_Models_ServiceTrade_Options::get_timezone();
		$timezone_identifiers = DateTimeZone::listIdentifiers(DateTimeZone::ALL);

		// Set the default timezone to the US
		if(empty($timezone)) {
			$timezone = "America/New_York";
		}

		?>
			<fieldset>
				<label>
					<select type="email"
									name="<?php echo 'servicetrade_portal_timezone' ?>"
									id="<?php echo 'servicetrade_portal_timezone' ?>"
									aria-describedby="timezone-description">

						<?php foreach($timezone_identifiers as $timezone_identifier )  { ?>
						<?php
							if($timezone_identifier == $timezone) {
							?>
								<option value="<?php echo($timezone_identifier); ?>" selected><?php echo($timezone_identifier); ?></option>
							<?php } else { ?>
								<option value="<?php echo($timezone_identifier); ?>"><?php echo($timezone_identifier); ?></option>
								<?php } ?>

						<?php } ?>

					</select>
					<?php _e( 'Your timezone.', 'servicetrade' ); ?>
				</label>
			</fieldset>
		<?php
	}

	/**
	 * Render the warning for invalid/missing API credentials
	 *
	 * @since  2.0.4
	 */
	public function api_credentials_notice__error() {
	    $currentScreen = get_current_screen();
	    $currentAccessToken = Servicetrade_Models_API_Credentials::get_access_token();
	    $currentUsername = Servicetrade_Models_API_Credentials::get_username();
		 $currentPassword = Servicetrade_Models_API_Credentials::get_password();

		 $hasCredentials = $currentUsername && $currentPassword;

	    // only show API credentials warnings on portal admin pages
		 $isPortalSettingsPage = ( $currentScreen && $currentScreen->parent_base == 'service-trade-admin' );

		 if ( $isPortalSettingsPage ) {

		  if ( !$hasCredentials ) {
	   	?>
	        <div class="notice notice-warning is-dismissible">
	   	     <p><?php _e( 'WARNING: You must provide ServiceTrade API credentials.', 'servicetrade' ); ?></p>
	        </div>
         <?php

		  } else if ( !$currentAccessToken ) {
	   	?>
	        <div class="notice notice-error is-dismissible">
	   	     <p><?php _e( 'ERROR: ServiceTrade API credentials are invalid.', 'servicetrade' ); ?></p>
	        </div>
	   	<?php
		  }

		 }
	}


	/**
	 * Sets up the admin menu
	 *
	 * @since    2.0.0
	 */
	public function setup_admin_menu(){
		add_menu_page( 'ServiceTrade Admin',
									 'ServiceTrade',
									 'manage_options',
									 'service-trade-admin',
									 array($this, 'render_settings'),
									 null,2);

		add_submenu_page(
						'service-trade-admin',
						'Users',
						'Users',
						'manage_options',
						'service-trade-admin-users',
						array($this, 'render_users'));
		add_submenu_page(
						'service-trade-admin',
						'Add User',
						'Add User',
						'manage_options',
						'service-trade-admin-add-user',
						array($this, 'render_add_user'));

		add_submenu_page(
						'service-trade-admin',
						'Settings',
						'Settings',
						'manage_options',
						'service-trade-admin-settings',
						array($this, 'render_settings'));
	}

	function render_settings() {
		include_once SERVICETRADE_PORTAL_PLUGIN_ROOT_DIR . 'admin/partials/plugin-servicetrade-portal-display-settings.php';
	}

	function render_users() {
		$controller = new Servicetrade_Service_Portal_Admin_Page_Users_Controller();
		$controller->render();
	}

	function render_add_user() {
		$controller = new Servicetrade_Service_Portal_Admin_Page_Add_User_Controller();
		$controller->render();
	}


	public function register_rest_controllers() {
		require_once SERVICETRADE_PORTAL_PLUGIN_ROOT_DIR . 'admin/rest/class-admin-routes-controller.php';

		$routes = new Servicetrade_Service_Portal_Admin_Routes_Controller();
		$routes->register_routes();
	}


}

