<?php

require_once SERVICETRADE_PORTAL_PLUGIN_ROOT_DIR . 'includes/class-servicetrade-action.php';
require_once SERVICETRADE_PORTAL_PLUGIN_ROOT_DIR . 'includes/models/class-servicetrade-options.php';

/**
 * Returns the list of users that requested access to the service trade portal.
 *
 * An action class.
 *
 * @link       http://www.servicetrade.com
 * @since      2.0.0
 * @package    Servicetrade_Service_Portal
 * @subpackage Servicetrade_Service_Portal/public/rest/actions
 * @author     ServiceTrade <support@servicetrade.com>
 */
class Servicetrade_Service_Portal_Admin_Actions_Rest_Users_Access_Requested_Get_Action extends Servicetrade_Action {

  function run() {
    $args = array(
      'fields' => 'all_with_meta',
      'meta_query' => 
          array(
              array(
                  'key'     => 'servicetrade_portal_user_status',
                  'value'   => 'requested',
                  'compare' => '='
              )
          )
    );
    $users_query = new WP_User_Query( $args );
    $results = $users_query->get_results();

    if(!$results) {
      return new WP_REST_Response( array('users' => array()), 200 );
    } else {
      $mappedResults = array();
      foreach ($results as $result) {
        $mappedResults[] = array(
          'id'     => $result->data->ID,
          'email'  => $result->data->user_email,
          'username'  => $result->data->user_login,
          'displayName'   => $result->data->display_name,
          'roles'  => $result->roles,
          'status' => $result->servicetrade_portal_user_status, // Meta stuff
          'canRequestService' =>  Servicetrade_Models_ServiceTrade_Options::get_request_service() , // json_decode($result->servicetrade_portal_can_request_service),
          'canSetRequestService' => Servicetrade_Models_ServiceTrade_Options::get_request_service(),
          'canViewQuotes' => true, // json_decode($result->servicetrade_portal_can_view_quotes),
          'canViewDeficiencies' => true, // json_decode($result->servicetrade_portal_can_view_deficiencies),
          'canViewPrices' => true // json_decode($result->servicetrade_portal_can_view_prices)
        );
      }


      return new WP_REST_Response( array( 'users' => $mappedResults ), 200 );
    }
  } 
}
