<?php

require_once SERVICETRADE_PORTAL_PLUGIN_ROOT_DIR . 'includes/class-servicetrade-action.php';
require_once SERVICETRADE_PORTAL_PLUGIN_ROOT_DIR . 'includes/class-servicetrade-user-meta.php';

/**
 * Accepts a user request.
 *
 * An action class.
 *
 * @link       http://www.servicetrade.com
 * @since      2.0.0
 * @package    Servicetrade_Service_Portal
 * @subpackage Servicetrade_Service_Portal/public/rest/actions
 * @author     ServiceTrade <support@servicetrade.com>
 */
class Servicetrade_Service_Portal_Admin_Actions_Rest_Users_User_Id_Access_Requested_Accept_Post_Action extends Servicetrade_Action {
  private $user_id;
  private $can_request_service;
  private $can_view_quotes;
  private $can_view_deficiencies;
  private $can_view_prices;
  
  public function __construct($user_id,
                              $can_request_service,
                              $can_view_quotes,
                              $can_view_deficiencies,
                              $can_view_prices ) {
    $this->user_id = $user_id;
    $this->can_request_service = $can_request_service;
    $this->can_view_quotes = $can_view_quotes;
    $this->can_view_deficiencies = $can_view_deficiencies;
    $this->can_view_prices = $can_view_prices;
  }

  function run() {

    Servicetrade_User_Meta::set_user_meta_data($this->user_id , 
      "active",
      $this->can_request_service,
      $this->can_view_quotes,
      $this->can_view_deficiencies,
      $this->can_view_prices);

    $userData = get_userdata($this->user_id);
    $user_email = $userData->user_email;

    $siteName = get_bloginfo('name');
    $siteUrl = get_bloginfo('url');

    $title = "Service portal access is available!";
    $message = "The account you requested for the service portal at {$siteName} ({$siteUrl}) has been enabled.  You may log in at {$siteUrl}.";

    if ( $message && !wp_mail($user_email, $title, $message) ) {
      return new WP_Error( 'servicetrade_cant_send_accepted_email', 'The email could not be sent.', 
                          array( 'status' => 422, 
                          'errors' => array('login' => 'The email could not be sent.'
      ) ) );
    }

    return new WP_REST_Response( array(), 200 );
  } 
}
