<?php

require_once SERVICETRADE_PORTAL_PLUGIN_ROOT_DIR . 'includes/models/class-api-credentials.php';
require_once SERVICETRADE_PORTAL_PLUGIN_ROOT_DIR . 'includes/models/class-servicetrade-options.php';
require_once SERVICETRADE_PORTAL_PLUGIN_ROOT_DIR . 'vendor/servicetrade/servicetrade-php/Servicetrade.php';


class Servicetrade_Wrapper extends Servicetrade
{
  public $lastStatus = null; 

  public static function hasCredentials() {
    $username = Servicetrade_Models_API_Credentials::get_username();
    $password = Servicetrade_Models_API_Credentials::get_password();

    return !(empty($username) || empty($password));
  }
  /**
   * This needs to be called once to set the access token,
   * either through the stored token or by aquiring it using
   * the username and password.
   */
  public function connect() {
    $username = Servicetrade_Models_API_Credentials::get_username();
    $password = Servicetrade_Models_API_Credentials::get_password();
    $current_access_token = Servicetrade_Models_API_Credentials::get_access_token();

    // error_log('Servicetrade_Wrapper::connect - current access token  ' . $current_access_token);

    if(!empty($current_access_token)) {
      // error_log('Servicetrade_Wrapper::connect - current access token not empty');
      $this->connectWithAuthId($current_access_token);
    } else {
      // error_log('Servicetrade_Wrapper::connect - password connect');
      $this->connectWithCredentials($username, $password);
      // error_log('Servicetrade_Wrapper::connect - password connect complete -saving token');
      Servicetrade_Models_API_Credentials::set_access_token($this->getAuthId());
    }
  }

  /**
   * Reconnects to the API.
   */
  protected function reconnect() {
    // error_log('Reconnecting...');
    Servicetrade_Models_API_Credentials::set_access_token(null);

    if(static::hasCredentials()) {
      // error_log('We have credentials...');
      $username = Servicetrade_Models_API_Credentials::get_username();
      $password = Servicetrade_Models_API_Credentials::get_password();
      
      $this->connectWithCredentials($username, $password);
      // error_log('Token' . $this->getAuthId() );
      Servicetrade_Models_API_Credentials::set_access_token($this->getAuthId());
    }
  }


  /**
	 * Connect using the authId
	 */
  protected function connectWithAuthId($authId) {
    $this->authId = $authId;
  }

  /**
	 * Connect using the authId
	 */
  protected function connectWithCredentials($username,$password) {
    $this->authenticate($username, $password);
  }

  /**
   * Overriding the constructor,
   * we DO NOT call the base constructor.
   */
  public function __construct ($baseUrl)
  {
    if ($baseUrl) {
			$this->baseUrl = $baseUrl;
		}
  }

  	/**
	 * Make a GET request to the API
	 *
	 * @param string $path   API path, excluding the base path
	 * @param array $query   Query params
	 * @return array
	 */
	public function get($path, $query=array())
	{
    // error_log('GET ' . $path);
    // $this->authId = "dummy";
    $result = parent::get( $path, $query);
 
    // error_log('GET ' . $path . ' STATUS '. $this->lastStatus);
 
    if(null === $result && 
       401 === (int)$this->lastStatus) {
      $this->reconnect();
      $result = parent::get( $path, $query);
    }
    return $result;
	}

	/**
	 * Make a POST request to the API
	 *
	 * @param string $path   API path, excluding the base path
	 * @param array $params  POST Params key/value array
	 * @param array $query   Query params
	 * @return array
	 */
	public function post($path, $params, $query=array())
	{
		$result = parent::post( $path, $params,$query);
    if(null === $result &&
       401 === (int)$this->lastStatus) {
      $this->reconnect();
      $result = parent::post( $path, $params,$query);
    }
    return $result;
  }

  // NOTE -> PUT/DELETE not implemented

  	/*
	 * Authenticate with API and return a session id
	 *
	 * @param string $user
	 * @param string $password
	 * return string session id
	 */
	protected function authenticate($user, $password)
	{
    // error_log('Servicetrade_Wrapper::authenticate - called');

    $data = array(
			'username' => $user,
			'password' => $password,
		);

    // WARNING - OVERRIDDEN BECAUSE OTHERWISE WE GET A 
    // CIRCULAR LOOP BELOW
    // error_log('Servicetrade_Wrapper::authenticate - pre post');
		$result = parent::post('/auth', $data);
    // error_log('Servicetrade_Wrapper::authenticate - post post');
    // error_log(print_r($result, true));
    // error_log(print_r($result['authToken'], true));
		$this->authId = isset($result['authToken']) ? $result['authToken'] : null;
	}

}

