<?php

require_once SERVICETRADE_PORTAL_PLUGIN_ROOT_DIR . 'includes/class-servicetrade-action.php';
require_once SERVICETRADE_PORTAL_PLUGIN_ROOT_DIR . 'includes/class-servicetrade-helper-sideloadmapper.php';
require_once SERVICETRADE_PORTAL_PLUGIN_ROOT_DIR . 'includes/class-servicetrade-helper-invoices.php';
require_once SERVICETRADE_PORTAL_PLUGIN_ROOT_DIR . 'public/rest/class-current-timezone.php';
require_once SERVICETRADE_PORTAL_PLUGIN_ROOT_DIR . 'public/rest/class-public-rest-action.php';

/**
 * Returns ALL jobs
 *
 * An action class.
 *
 * @link       http://www.servicetrade.com
 * @since      2.0.0
 * @package    Servicetrade_Service_Portal
 * @subpackage Servicetrade_Service_Portal/public/rest/actions
 * @author     ServiceTrade <support@servicetrade.com>
 * @example
 *						 curl http://localhost:8888/wp-json/servicetrade-service-portal/v2/public/jobs/recently-completed
 */
class Servicetrade_Service_Portal_Public_Actions_Rest_Jobs_Get_Action extends Servicetrade_Service_Portal_Public_Rest_Action {

  private $filterLocationId;
  private $filterServiceLineId;
  private $filterJobType;

	public function __construct($filterLocationId,
															$filterServiceLineId,
                              $filterJobType) {
    $this->filterLocationId = $filterLocationId;
		$this->filterServiceLineId = $filterServiceLineId;
    $this->filterJobType = $filterJobType;
  }

	function run() {
		$timezone = Servicetrade_Service_Portal_Public_Current_Timezone::timezone();
    $user = $this->get_authenticated_user();
    if(null === $user) {
			return $this->create_rest_error_not_authorized();
    }

		$contactId = $user->getContactId();

    $windowBeginsAfter = strtotime( '-90 days' );
		$windowBeginsBefore = strtotime( 'midnight today' );
		$windowBeginsTodayStart = $timezone->todayStart;
		$windowBeginsTodayEnd = $timezone->todayEnd;

		$contextKey = null;
		$contextVals = [];
		if ( $user->isCompanyOnlyContact() ) {
			$contextVals = $user->getContactCompanies( true );
			$contextKey = 'customerId';
		} else {
			$contextVals = $user->getContactLocations( true );
			$contextKey = 'locationId';
		}

		if (!$contextKey || !$contextVals) {
			$result =  array(
				jobs => [],
				status => 'no_associated_contacts'
			);
			return new WP_REST_Response( $result, 200 );

			// return new WP_Error( 'servicetrade_failed_request' ,'The plugin could not retrieve data from the servicetrade API', array( 'status' => 503 ) );
		}
		$query = [
//			'status' => 'completed',
			'windowBeginsAfter' => $windowBeginsAfter,
//			'windowBeginsBefore' => $windowBeginsBefore,
			$contextKey => $contextVals,
		];

		 $recentJobs = $this->get_st_appointments_by_query( $query );

		/*
		Result Mapping - this should be refactored into a common base class (eventually)
		*/

		// $mappedResults = array();
		$todayResults = array();
		$upcomingResults = array();
		$recentlyCompletedResults = array();

		foreach ($recentJobs as $result) {

		 $showCanceledAppointments = Servicetrade_Models_ServiceTrade_Options::get_show_canceled_appointments();
		 $canceledStatuses = ['canceled_by_vendor', 'canceled_by_customer'];
		 if (!$showCanceledAppointments && in_array($result['status'], $canceledStatuses)) {
			continue;
		 }

			$mappedServiceLines = array();
			$mappedServiceDescriptions = array();

			foreach ($result['serviceRequests'] as $serviceRequest) {
				if ($serviceRequest['description'] && array_intersect(['public', 'customer'], $serviceRequest['visibility'])) {
					$mappedServiceDescriptions[] = $serviceRequest['description'];
				}
				if( isset($serviceRequest['serviceLine'])) {
					$mappedServiceLines[$serviceRequest['serviceLine']['id']] = array(
						'id' => $serviceRequest['serviceLine']['id'],
						'name' => $serviceRequest['serviceLine']['name'],
					);
				}
			}

			// we use SL IDs to ensure a unique list of SLs, but now the array needs to be reindexed
			$mappedServiceLines = array_values($mappedServiceLines);

			list($invoiceAmount, $invoices) = Servicetrade_Helper_Invoices::organize_invoices($result['job']['invoices'], $contactId);

			$isOnsite = false;
			$isEnroute = false;
			$onsiteStartTime = null;
			$enrouteStartTime = null;

			if($result['status'] === 'scheduled') {
				if(! empty($result['clockEvents']) ) {
					foreach ($result['clockEvents'] as $clock_event) {
						if($clock_event['type'] === 'clock-in' &&
							$clock_event['activity'] === 'onsite') {
								$isOnsite = true;
								$onsiteStartTime = $clock_event['time'];
						} else if($clock_event['type'] === 'clock-out' &&
							$clock_event['activity'] === 'onsite') {
							$isOnsite = false;
						} else if($clock_event['type'] === 'clock-in' &&
							$clock_event['activity'] === 'enroute') {
								$isEnroute = true;
								$enrouteStartTime = $clock_event['time'];
						} else if($clock_event['type'] === 'clock-out' &&
							$clock_event['activity'] === 'enroute') {
							$isEnroute = false;
						}
					}
				}
			}



			// FILTER

			if(!empty($this->filterLocationId)) {
				if($this->filterLocationId != $result['job']['location']['id']) {
					continue;
				}
			}

			if(!empty($this->filterJobType)) {
				if($this->filterJobType != $result['job']['type']) {
					continue;
				}
			}

			if(!empty($this->filterServiceLineId)) {
				$foundServiceLine = false;

				foreach($mappedServiceLines as $serviceLine) {
					if($this->filterServiceLineId == $serviceLine['id']) {
						$foundServiceLine = true;
					}
				}

				if(!$foundServiceLine) {
					continue;
				}
			}

			$serviceLinkData = ['jobId' => $result['job']['id'], 'contactId' => $contactId];
			$encryptedServiceLinkData = Servicetrade_Service_Portal_Public_Authenticated_User::encryptUserData($serviceLinkData);

			$jobDescription = in_array('description', $result['job']['visibility']) ? $result['job']['description'] : '';
			if (!$jobDescription) {
				$jobDescription = implode('<br />', $mappedServiceDescriptions);
			}

			$isCanceled = strpos($result['status'], 'canceled') !== false;

			// $mappedResults[]
			$mappedResult = array(
				'id' => $result['id'],
				'uri' => $result['uri'],
				'serviceLinkData' => $encryptedServiceLinkData,
				'description' => $jobDescription,
				'locationName' => $result['job']['location']['name'],
				'locationId' => $result['job']['location']['id'],
				'windowStart' => $result['windowStart'],
				'jobId' => $result['job']['id'],
				'jobType' => $result['job']['type'],
				'jobNumber' => $result['job']['number'],
				'jobTypeDisplay' => ucwords( str_replace("_", " ", $result['job']['type'])),
				'status' => $result['status'],
				'serviceLines' => $mappedServiceLines,
				'invoiceAmount' => $invoiceAmount,
				'invoices' => $invoices,
				'isCanceled' => $isCanceled,
				'isOnsite' => $isOnsite,
				'isEnroute' => $isEnroute
			);

			/**
			 * Finally we filter out the results and put it in the correct
			 * buckets
			 */

			if($result['status'] === 'completed' &&
	 			$result['windowStart'] < $windowBeginsBefore) {
				$recentlyCompletedResults[] = $mappedResult;
			} else if ($result['windowStart'] >= $windowBeginsTodayStart &&
				$result['windowStart'] < $windowBeginsTodayEnd) {
				$todayResults[] = $mappedResult;
			} else if ($result['windowStart'] >= $windowBeginsTodayEnd) {
				$upcomingResults[] = $mappedResult;
			}
		}


    $result =  array(
			"today" => $todayResults,
			"upcoming" => $upcomingResults,
			"recently_completed" => $recentlyCompletedResults,
		);
		return new WP_REST_Response( $result, 200 );
	}

		/**
	 * Find appointments by a query from ServiceTrade API
	 * @since     1.0.0
	 * @return    array    Array of ServiceTrade appointments arrays
	 */
	 public function get_st_appointments_by_query($query=[]) {

		 $queryChunks = [];
		 if(isset($query['locationId']) && is_array($query['locationId'])) {
			// break location ID queries into 100-location chunks so our request URL isn't too long
			$locationIdChunks = array_chunk($query['locationId'], 100);
			foreach($locationIdChunks as $locationIdChunk) {
				$locationParam = array('locationId' => implode(',', $locationIdChunk));
				$queryChunks[] = array_merge($query, $locationParam);
			}
		 } else if(isset($query['customerId']) && is_array($query['customerId'])) {
			$customerParam = array('customerId' => implode(',',$query['customerId']));
			$queryChunks[] = array_merge($query, $customerParam);
		}

		$results = [];
		foreach($queryChunks as $queryChunk) {
			$params = http_build_query($queryChunk);
			$params .= '&_sideload=appointment.job,job.location,job.deficiencies,appointment.clockEvents,appointment.serviceRequests,serviceRequest.serviceLine,job.invoices';


			$data = $this->st_api()->get( '/appointment?'.$params );
			if ( $data['appointments'] ) {
				  $mapParams = [
					 'job'             => 'jobs',
					 'location'        => 'locations',
					 'clockEvents'     => 'clockEvents',
					 'serviceRequests' => 'serviceRequests',
					 'serviceLine'     => 'serviceLines',
					 'deficiencies'    => 'deficiencies',
					 'invoices'        => 'invoices',
				  ];
				  $mappedData = Servicetrade_Helper_Sideloadmapper::map_sideloaded_data( $data, 'appointments', $mapParams );
				  $results = array_merge($results, $mappedData);
			}
		}

		return $results;
	}
}
