<?php

require_once SERVICETRADE_PORTAL_PLUGIN_ROOT_DIR . 'includes/class-servicetrade-action.php';
require_once SERVICETRADE_PORTAL_PLUGIN_ROOT_DIR . 'public/rest/class-public-rest-action.php';

/**
 * Returns the assets belonging to a location.
 *
 * @example
 * http://localhost:8888/wp-json/servicetrade-service-portal/v2/public/locations/1739987/assets
 *
 * @link       http://www.servicetrade.com
 * @since      2.0.0
 * @package    Servicetrade_Service_Portal
 * @subpackage Servicetrade_Service_Portal/public/rest/actions
 * @author     ServiceTrade <support@servicetrade.com>
 */
class Servicetrade_Service_Portal_Public_Actions_Rest_Location_Assets_Get_Action extends Servicetrade_Service_Portal_Public_Rest_Action {
  private $id;

  function __construct($id) {
    assert($id !== null, "An id is required.");
    $this->id = $id;
  }

  function run() {
    $user = $this->get_authenticated_user();
    if(null === $user) {
			return $this->create_rest_error_not_authorized();
    }

    $apiResult = $this->api_get($this->build_location_assets_query());
    if(!isset( $apiResult['assets'] )) {
      return $this->create_rest_error_failed_request();
    }
    $assets = $apiResult['assets'];

    $mappedResults = array();
    
    foreach ($assets as $result) {
      $mappedResults[] = array(
        'id' => $result['id'],
        'uri' => $result['uri'],
        'name' => $result['name'],
        'assetType' => $result['display']
      );
    }

    $restResponse =  array(
      "assets" => $mappedResults
    );
    return $this->create_rest_success($restResponse);
  }

  protected function build_location_assets_query() {
		$query = array( 
			'locationId' => $this->id,
		);
    return '/asset?' . http_build_query($query);
  }
  
}
