<?php

require_once SERVICETRADE_PORTAL_PLUGIN_ROOT_DIR . 'includes/class-servicetrade-action.php';
require_once SERVICETRADE_PORTAL_PLUGIN_ROOT_DIR . 'public/rest/class-public-rest-action.php';

/**
 * THIS IS NOT REALLY A REST ENDPOINT.  It simply redirects the browser to the service link page for
 * a given record (job + contact, or quote + contact)
 *
 * @example
 * http://localhost:8888/wp-json/servicetrade-service-portal/v2/public/redirect/sA3Xge+L07fFsHqQwcakihtUY5sXG+mFib/3b0/OMRk=
 *
 * @link       http://www.servicetrade.com
 * @since      2.0.0
 * @package    Servicetrade_Service_Portal
 * @subpackage Servicetrade_Service_Portal/public/rest/actions
 * @author     ServiceTrade <support@servicetrade.com>
 */
class Servicetrade_Service_Portal_Public_Actions_Rest_Redirect_Get_Action extends Servicetrade_Service_Portal_Public_Rest_Action {
  function __construct($token) {
	  assert($token !== null, "A token is required.");
    $this->token = $token;
  }

  function run() {
		if(null === $this->token) {
			return new WP_Error( 'servicetrade_failed_request' ,'The plug-in needs a token to retrieve data from the servicetrade API.', array( 'status' => 403 ) );
		}

		$userData = Servicetrade_Service_Portal_Public_Authenticated_User::decryptUserData($this->token);
		$params = $userData ? http_build_query($userData) : '';

		$data = $this->st_api()->get( '/token?' . $params );
		if ($data && $data['token']) {
			if ($userData['jobId']) {
				$url = SERVICETRADE_WEB_URL . '/customer/jobsummary?id=' . $data['token'];
			} else if ($userData['quoteId']) {
				$url = SERVICETRADE_WEB_URL . '/quoteview/view?id=' . $data['token'];
			} else if ($userData['invoiceId']) {
				$url = SERVICETRADE_WEB_URL . '/customer/invoice?id=' . $data['token'];
			}
			header('Location: ' . $url);
			exit();
		}
		return new WP_Error( 'servicetrade_failed_request' ,'The plug-in could not generate a token from the servicetrade API.', array( 'status' => 403 ) );

  }
}
