<?php

require_once SERVICETRADE_PORTAL_PLUGIN_ROOT_DIR . 'includes/class-servicetrade-action.php';
require_once SERVICETRADE_PORTAL_PLUGIN_ROOT_DIR . 'public/rest/class-public-rest-action.php';

/**
 * Creates a user account and requests access.
 *
 * @link       http://www.servicetrade.com
 * @since      2.0.0
 * @package    Servicetrade_Service_Portal
 * @subpackage Servicetrade_Service_Portal/public/rest/actions
 * @author     ServiceTrade <support@servicetrade.com>
 */
class Servicetrade_Service_Portal_Public_Actions_Rest_User_Request_Access_Post_Action extends Servicetrade_Service_Portal_Public_Rest_Action {

  private $name;
  private $email;
  private $password;

  public function __construct($name,$email,$password ) {
    $this->name = $name;
    $this->email =$email;
    $this->password = $password;
  }

  function run() {
    // Validate basics

    if(!$this->name) {
      return new WP_Error( 'servicetrade_no_user', 'Please provide a name', array( 'status' => 422 ) );
	 }

    if(!$this->password) {
      return new WP_Error( 'servicetrade_no_password', 'Please provide a password', array( 'status' => 422 ) );
	 }

    if(!$this->email) {
      return new WP_Error( 'servicetrade_no_email', 'Please provide an email', array( 'status' => 422 ) );
	 }

    if(username_exists($this->name)) {
      return new WP_Error( 'servicetrade_user_exists', 'This name already exists', array( 'status' => 422 ) );
    }

    if(email_exists($this->email)) {
      return new WP_Error( 'servicetrade_email_exists', 'This email already exists', array( 'status' => 422 ) );
    }

    $user_id = wp_create_user( $this->name, $this->password, $this->email );

    add_user_meta(
      $user_id,
      "servicetrade_portal_user_status",
      "requested",
      false
	);

	 $siteName = get_bloginfo('name');
	 $siteUrl = get_bloginfo('url');

	 // send admin to customer
	 $customerEmail = $this->email;
	 $customerSubject = "Service portal access request";
	 $customerMessage = "You have requested an account for the service portal at {$siteName} ({$siteUrl}).  You will receive a response shortly.";
    wp_mail( $customerEmail, $customerSubject, $customerMessage );

	 // send email to admin
	 $adminEmail = Servicetrade_Models_ServiceTrade_Options::get_admin_email();
	 $adminSubject = "ServiceTrade service portal: Customer account request ({$siteName})";
	 $adminMessage = "{$this->name} ({$this->email}) has requested access to the ServiceTrade service portal at " . $siteName . " (" . $siteUrl . ").";
	 $adminMessage .= "\n\nTo approve this request, log into the site as a WordPress administrator and approve this user in the ServiceTrade portal plugin user list.";
    wp_mail( $adminEmail, $adminSubject, $adminMessage );


    $result =  array(

    );
    return new WP_REST_Response( $result, 201 );
  }
}
